@@ -39,10 +39,6 @@
 #include <retro_inline.h>
 #include <retro_common_api.h>
 #include <retro_miscellaneous.h>
-#include <gfx/math/matrix_4x4.h>
-#include <gfx/scaler/scaler.h>
-#include <rthreads/rthreads.h>
-#include <formats/image.h>
 
 #include <libretro.h>
 #include <libretro_vulkan.h>
@@ -110,10 +106,7 @@
    manager->count = 0; \
 }
 
-#define VK_MAP_PERSISTENT_TEXTURE(device, texture) \
-{ \
-   vkMapMemory(device, texture->memory, texture->offset, texture->size, 0, &texture->mapped); \
-}
+#define VK_MAP_PERSISTENT_TEXTURE(device, texture) vkMapMemory(device, texture->memory, texture->offset, texture->size, 0, &texture->mapped)
 
 #define VULKAN_PASS_SET_TEXTURE(device, set, _sampler, binding, image_view, image_layout) \
 { \
@@ -328,28 +321,6 @@ enum vk_texture_flags
    VK_TEX_FLAG_MIPMAP                       = (1 << 2)
 };
 
-#ifdef VULKAN_HDR_SWAPCHAIN
-
-#ifndef VKALIGN
-#ifdef _MSC_VER
-#define VKALIGN(x) __declspec(align(x))
-#else
-#define VKALIGN(x) __attribute__((aligned(x)))
-#endif
-#endif
-
-typedef struct VKALIGN(16)
-{
-   math_matrix_4x4   mvp;
-   float             contrast;         /* 2.0f    */
-   float             paper_white_nits; /* 200.0f  */
-   float             max_nits;         /* 1000.0f */
-   float             expand_gamut;     /* 1.0f    */
-   float             inverse_tonemap;  /* 1.0f    */
-   float             hdr10;            /* 1.0f    */
-} vulkan_hdr_uniform_t;
-#endif /* VULKAN_HDR_SWAPCHAIN */
-
 typedef struct vulkan_context
 {
    slock_t *queue_lock;
@@ -426,48 +397,6 @@ struct vulkan_display_surface_info
    unsigned refresh_rate_x1000;
 };
 
-struct vk_color
-{
-   float r, g, b, a;
-};
-
-struct vk_vertex
-{
-   float x, y;
-   float tex_x, tex_y;
-   struct vk_color color;        /* float alignment */
-};
-
-struct vk_image
-{
-   VkImage image;                /* ptr alignment */
-   VkImageView view;             /* ptr alignment */
-   VkFramebuffer framebuffer;    /* ptr alignment */
-   VkDeviceMemory memory;        /* ptr alignment */
-};
-
-struct vk_texture
-{
-   VkDeviceSize memory_size;     /* uint64_t alignment */
-
-   void *mapped;
-   VkImage image;                /* ptr alignment */
-   VkImageView view;             /* ptr alignment */
-   VkBuffer buffer;              /* ptr alignment */
-   VkDeviceMemory memory;        /* ptr alignment */
-
-   size_t offset;
-   size_t stride;
-   size_t size;
-   uint32_t memory_type;
-   unsigned width, height;
-
-   VkImageLayout layout;         /* enum alignment */
-   VkFormat format;              /* enum alignment */
-   enum vk_texture_type type;
-   uint8_t flags;
-};
-
 struct vk_buffer
 {
    VkDeviceSize size;      /* uint64_t alignment */
@@ -516,173 +445,6 @@ struct vk_descriptor_manager
    unsigned num_sizes;
 };
 
-struct vk_per_frame
-{
-   struct vk_texture texture;          /* uint64_t alignment */
-   struct vk_texture texture_optimal;
-   struct vk_buffer_chain vbo;         /* uint64_t alignment */
-   struct vk_buffer_chain ubo;
-   struct vk_descriptor_manager descriptor_manager;
-
-   VkCommandPool cmd_pool; /* ptr alignment */
-   VkCommandBuffer cmd;    /* ptr alignment */
-};
-
-struct vk_draw_quad
-{
-   struct vk_texture *texture;
-   const math_matrix_4x4 *mvp;
-   VkPipeline pipeline;          /* ptr alignment */
-   VkSampler sampler;            /* ptr alignment */
-   struct vk_color color;        /* float alignment */
-};
-
-struct vk_draw_triangles
-{
-   const void *uniform;
-   const struct vk_buffer_range *vbo;
-   struct vk_texture *texture;
-   VkPipeline pipeline;          /* ptr alignment */
-   VkSampler sampler;            /* ptr alignment */
-   size_t uniform_size;
-   unsigned vertices;
-};
-
-typedef struct vk
-{
-   vulkan_filter_chain_t *filter_chain;
-   vulkan_context_t *context;
-   void *ctx_data;
-   const gfx_ctx_driver_t *ctx_driver;
-   struct vk_per_frame *chain;
-   struct vk_image *backbuffer;
-#ifdef VULKAN_HDR_SWAPCHAIN
-   VkRenderPass readback_render_pass;
-   struct vk_image main_buffer;
-   struct vk_image readback_image;
-#endif /* VULKAN_HDR_SWAPCHAIN */
-
-   unsigned video_width;
-   unsigned video_height;
-
-   unsigned tex_w, tex_h;
-   unsigned out_vp_width;
-   unsigned out_vp_height;
-   unsigned rotation;
-   unsigned num_swapchain_images;
-   unsigned last_valid_index;
-
-   video_info_t video;
-
-   VkFormat tex_fmt;
-   math_matrix_4x4 mvp, mvp_no_rot, mvp_menu; /* float alignment */
-   VkViewport vk_vp;
-   VkRenderPass render_pass;
-   struct video_viewport vp;
-   float translate_x;
-   float translate_y;
-   struct vk_per_frame swapchain[VULKAN_MAX_SWAPCHAIN_IMAGES];
-   struct vk_image backbuffers[VULKAN_MAX_SWAPCHAIN_IMAGES];
-   struct vk_texture default_texture;
-
-   /* Currently active command buffer. */
-   VkCommandBuffer cmd;
-   /* Staging pool for doing buffer transfers on GPU. */
-   VkCommandPool staging_pool;
-
-   struct
-   {
-      struct scaler_ctx scaler_bgr;
-      struct scaler_ctx scaler_rgb;
-      struct vk_texture staging[VULKAN_MAX_SWAPCHAIN_IMAGES];
-   } readback;
-
-   struct
-   {
-      struct vk_texture *images;
-      struct vk_vertex *vertex;
-      unsigned count;
-   } overlay;
-
-   struct
-   {
-      VkPipeline alpha_blend;
-      VkPipeline font;
-      VkPipeline rgb565_to_rgba8888;
-#ifdef VULKAN_HDR_SWAPCHAIN
-      VkPipeline hdr;
-      VkPipeline hdr_to_sdr; /* for readback */
-#endif /* VULKAN_HDR_SWAPCHAIN */
-      VkDescriptorSetLayout set_layout;
-      VkPipelineLayout layout;
-      VkPipelineCache cache;
-   } pipelines;
-
-   struct
-   {
-      VkPipeline pipelines[8 * 2];
-      struct vk_texture blank_texture;
-   } display;
-
-#ifdef VULKAN_HDR_SWAPCHAIN
-   struct
-   {
-      struct vk_buffer  ubo;
-      float             max_output_nits;
-      float             min_output_nits;
-      float             max_cll;
-      float             max_fall;
-   } hdr;
-#endif /* VULKAN_HDR_SWAPCHAIN */
-
-   struct
-   {
-      struct vk_texture textures[VULKAN_MAX_SWAPCHAIN_IMAGES];
-      struct vk_texture textures_optimal[VULKAN_MAX_SWAPCHAIN_IMAGES];
-      unsigned last_index;
-      float alpha;
-      bool dirty[VULKAN_MAX_SWAPCHAIN_IMAGES];
-   } menu;
-
-   struct
-   {
-      VkSampler linear;
-      VkSampler nearest;
-      VkSampler mipmap_nearest;
-      VkSampler mipmap_linear;
-   } samplers;
-
-   struct
-   {
-      const struct retro_vulkan_image *image;
-      VkPipelineStageFlags *wait_dst_stages;
-      VkCommandBuffer *cmd;
-      VkSemaphore *semaphores;
-      VkSemaphore signal_semaphore; /* ptr alignment */
-
-      struct retro_hw_render_interface_vulkan iface;
-
-      unsigned capacity_cmd;
-      unsigned last_width;
-      unsigned last_height;
-      uint32_t num_semaphores;
-      uint32_t num_cmd;
-      uint32_t src_queue_family;
-
-   } hw;
-
-   struct
-   {
-      uint64_t dirty;
-      VkPipeline pipeline; /* ptr alignment */
-      VkImageView view;    /* ptr alignment */
-      VkSampler sampler;   /* ptr alignment */
-      math_matrix_4x4 mvp;
-      VkRect2D scissor;    /* int32_t alignment */
-   } tracker;
-   uint32_t flags;
-} vk_t;
-
 bool vulkan_buffer_chain_alloc(const struct vulkan_context *context,
       struct vk_buffer_chain *chain, size_t len,
       struct vk_buffer_range *range);
